#include <opencv2/opencv.hpp>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/objdetect/objdetect.hpp>
#include <opencv2/ml/ml.hpp>

#include <iostream>
#include <string>
#include <vector>
#include <fstream>

//Get HOG features from images
static void getHog(const std::string &fileName, cv::HOGDescriptor &hog, std::vector<float> &feature){
	cv::Mat image = cv::imread(fileName.c_str(), CV_LOAD_IMAGE_GRAYSCALE);

	//Error checks
	if (image.empty()){
		std::cout<<"Image HOG calculation failed"<<std::endl;
		feature.clear();
		return;
	}

	resize(image,image, cv::Size(64,64));

    std::vector<cv::Point> locations;
    hog.compute(image, feature, cv::Size(8, 8), cv::Size(0, 0), locations);

	std::cout<<"HOG  size "<<feature.size()<< std::endl; 
	
    image.release();
}

int main()
{
	std::cout << "OpenCV SVN Training Demo"<<std::endl;

	//Set Number of samples
	int pos_imcount = 1000;
	int neg_imcount = 2000; 
	int hogFeaturesCount = 1764;  // (Hog Feature Size)


	int fileCount = 0;

	//Define Data
	cv::Mat training_data(pos_imcount+neg_imcount,hogFeaturesCount,CV_32FC1);

	//Define Labels
	cv::Mat training_labels(pos_imcount+neg_imcount, 1, CV_32FC1, cv::Scalar(-1.0));
	training_labels.rowRange(0, pos_imcount) = cv::Scalar(1.0);

	cv::HOGDescriptor hog;
	hog.winSize = cv::Size(64,64);
	std::vector<float> hogFeature;

	std::string line;

	//Positives
	std::ifstream positiveFile("./positives.txt");
	for (int i = 0; i < pos_imcount; i++) {

		std::getline(positiveFile, line);
		std::cout<<"Training Positive image "<<line<<std::endl;

		getHog(line, hog, hogFeature);

		for (int j=0; j < hogFeaturesCount; j++) {
			training_data.at<float>(i,j) = hogFeature.at(j);
		}

		fileCount++;
	}


	//Negatives
	std::ifstream negativeFile("./negatives.txt");
	for (int i = 0; i < neg_imcount; i++) {

		std::getline(negativeFile, line);
		std::cout<<"Training Negative image "<<line<<std::endl;

		getHog(line, hog, hogFeature);

		for (int j=0; j < hogFeaturesCount; j++) {
			training_data.at<float>(pos_imcount+i,j) = hogFeature.at(j);
		}

			fileCount++;
	}

//********************************

	CvSVMParams params;
	params.svm_type = CvSVM::C_SVC;
	params.kernel_type = CvSVM::LINEAR;
	params.term_crit   = cvTermCriteria(CV_TERMCRIT_ITER, 100, 1e-6);

	std::cout << "Training..."<<std::endl;

	CvSVM svm;
	svm.train(training_data, training_labels, cv::Mat(), cv::Mat(), params);

	svm.save("svm_test.dat");	// saving

	std::cout <<fileCount<<" Training Done..."<<std::endl;

//********************************

	fileCount = 0;
	float result;

	int correct = 0;
	int error = 0; 

	//Positives -- cotinue lodaing
	for (int i = 0; i < pos_imcount; i++) { 

		std::getline(positiveFile, line);
		std::cout<<"Testing Positive image "<<line<<std::endl;

		getHog(line, hog, hogFeature);

		cv::Mat data(1,hogFeaturesCount,CV_32FC1);

		for (int i=0; i < hogFeaturesCount; i++) {
			data.at<float>(0,i) = hogFeature.at(i);
		}

		result = svm.predict(data);

		if (result > 0) correct++;
		else error++;

		fileCount++;
	}


	//Negatives -- cotinue lodaing
	for (int i = 0; i < neg_imcount; i++) {

		std::getline(negativeFile, line);
		std::cout<<"Testing Negative image "<<line<<std::endl;

		getHog(line, hog, hogFeature);
		
		cv::Mat data(1,hogFeaturesCount,CV_32FC1);
		for (int i=0; i < hogFeaturesCount; i++) {
			data.at<float>(0,i) = hogFeature.at(i);
		}

		result = svm.predict(data);

		if (result < 0) correct++;
		else error++;

		fileCount++;
	}

	positiveFile.close();
	negativeFile.close();

	
	std::cout<<fileCount<<" files  - Correct: "<<correct<<" Wrong: "<<error<<" Succes: "<<correct/((correct+error)/100)<<"%"<<std::endl;

	return 0;
}

